package frc.robot.subsystems;

import com.revrobotics.AbsoluteEncoder;
import com.revrobotics.spark.SparkMax;
import com.revrobotics.spark.SparkLowLevel.MotorType;

import edu.wpi.first.math.controller.ElevatorFeedforward;
import edu.wpi.first.math.controller.PIDController;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.ElevatorConstants;

public class Elevator extends SubsystemBase {
    private SparkMax elevatorMotor1;

    private AbsoluteEncoder encoder;

    private PIDController pidController;

    private ElevatorFeedforward feedForward;
    
    public Elevator() {
        elevatorMotor1 = new SparkMax(
            ElevatorConstants.kElevatorMotorID, 
            MotorType.kBrushless
        );

        encoder = elevatorMotor1.getAbsoluteEncoder();

        pidController = new PIDController(
            ElevatorConstants.kPIDControllerP, 
            ElevatorConstants.kPIDControllerI, 
            ElevatorConstants.kPIDControllerD
        );

        feedForward = new ElevatorFeedforward(
            ElevatorConstants.kFeedForwardS, 
            ElevatorConstants.kFeedForwardG, 
            ElevatorConstants.kFeedForwardV
        );
    }

    public Command runElevator(double speed) {
        return run(() -> {
            elevatorMotor1.set(speed);
        });
    }
    
    public Command goToSetpoint(double setpoint, double timeout) {
        return run(() -> {
            double voltsOut = pidController.calculate(
                encoder.getPosition(),
                setpoint
            ) + feedForward.calculate(0);

            elevatorMotor1.setVoltage(voltsOut);
        }).until(pidController::atSetpoint).withTimeout(timeout);
    }
}
